from asyncio import sleep
import json
import re

import pyppeteer
from pyppeteer.errors import TimeoutError, PageError
from bs4 import BeautifulSoup

from app.settings import ACCOUNT_LOGIN, ACCOUNT_PASSWORD


class Parser:
    class __Parser:
        __login_url = 'https://login.aliexpress.com/'

        def __init__(self):
            self.__browser = None

        async def start_browser(self):
            self.__browser = await pyppeteer.launch({'args': ['--no-sandbox',
                                                              '--window-size=720,680',
                                                              '--shm-size=1gb',
                                                              '--enable-popup-blocking'],
                                                     'headless': True,
                                                     'waitUntil': ['load', 'domcontentloaded', 'networkidle0']})

        async def close_browser(self):
            await self.__browser.close()

        async def login(self):
            browser = await pyppeteer.connect({'browserWSEndpoint': self.__browser.wsEndpoint})
            login_page = await browser.newPage()
            try:
                await login_page.goto(url=self.__login_url,
                                      options={'waitUntil': ['load', 'domcontentloaded', 'networkidle0'],
                                               'timeout': 10000})
            except TimeoutError:
                pass

            await login_page.frames[2].type('input#fm-login-id', ACCOUNT_LOGIN)
            await login_page.frames[2].type('input#fm-login-password', ACCOUNT_PASSWORD)
            await login_page.frames[2].click('button.fm-button.fm-submit.password-login')
            await login_page.close()

        async def __get_product_html(self, url: str) -> str:
            browser = await pyppeteer.connect({'browserWSEndpoint': self.__browser.wsEndpoint})
            page = await browser.newPage()

            try:
                await page.goto(url=url, options={'waitUntil': ['load', 'domcontentloaded', 'networkidle0'],
                                                  'timeout': 10000})
            except TimeoutError:
                pass

            try:
                await page.click('a.next-dialog-close')
            except PageError:
                pass

            height = await page.evaluate("() => document.body.scrollHeight")
            cur_dist = 0
            while cur_dist < height:
                await page.evaluate("window.scrollBy(0, 600);")
                await sleep(0.1)
                cur_dist += 600

            try:
                await page.click('li[ae_object_type="specs"]')
            except PageError:
                pass

            html = await page.content()
            await page.close()
            return html

        async def __get_category_html(self, url: str) -> str:
            browser = await pyppeteer.connect({'browserWSEndpoint': self.__browser.wsEndpoint})
            page = await browser.newPage()

            try:
                await page.goto(url=url, options={'waitUntil': ['load', 'domcontentloaded', 'networkidle0'],
                                                  'timeout': 10000})
            except TimeoutError:
                pass

            height = await page.evaluate("() => document.body.scrollHeight")
            cur_dist = 0
            while cur_dist < height:
                await page.evaluate("window.scrollBy(0, 600);")
                await sleep(0.1)
                cur_dist += 600

            html = await page.content()
            await page.close()
            return html

        @staticmethod
        def __get_info_from_category_page(html: str) -> list:
            soup = BeautifulSoup(html, 'html.parser')
            all_products = soup.find_all('li', class_='list-item')

            if len(all_products) == 0:
                return []

            json_list = []
            for product in all_products:
                json = dict()
                json['id'] = product.find('div', class_='product-card')['data-product-id']
                json['title'] = product.find('a', class_='item-title').text
                json['price'] = product.find('span', class_='price-current').text.replace('\xa0', '')
                try:
                    json['url'] = 'https:' + product.find('div', class_='place-container').find('a')['href']
                except AttributeError:
                    json['url'] = 'https:' + product.find('div', class_='product-img').find('a')['href']

                json['img-url'] = 'https:' + product.find('img')['src'].replace('.jpg_220x220xz', '')
                json_list.append(json)
            return json_list

        @staticmethod
        def __form_variations(js: dict, s_obj: dict) -> dict:
            js['variations'] = []
            for name in s_obj['skuModule']['productSKUPropertyList']:
                if name['skuPropertyName'] == "Цвет":
                    js['variations'].append({'Name': name['skuPropertyName'],
                                             'Value': {},
                                             'Data': {"type": "color",
                                                      'colors': {},
                                                      'data': {}}})
                elif name['skuPropertyName'] == "Размер":
                    js['variations'].append({'Value': {},
                                             'Data': {}})

                for i in range(len(js['variations'])):
                    for item in name['skuPropertyValues']:
                        try:
                            js['variations'][i]['Value'].update(
                                {item['propertyValueId']: item['propertyValueName']})
                        except:
                            pass

                        try:
                            js['variations'][i]['Data']['colors'].update(
                                {item['propertyValueId']: item['skuPropertyImageSummPath']})
                            js['variations'][i]['Data']['data'][item['propertyValueId']] = dict()
                            js['variations'][i]['Data']['data'][item['propertyValueId']].update(
                                {'bigpic': item['skuPropertyImagePath']})
                        except KeyError:
                            pass

            js['variations_prices'] = []
            for price_obj in s_obj['skuModule']['skuPriceList']:
                js['variations_prices'].append({'vIds': price_obj['skuPropIds'],
                                                'price': price_obj['skuVal']['skuMultiCurrencyCalPrice']})

            return js

        def __get_info_from_product_page(self, html: str) -> dict:
            soup = BeautifulSoup(html, 'html.parser')

            js = dict()
            js['title'] = soup.find('div', class_='product-title').text
            js['price'] = soup.find('span', class_='product-price-value').text.replace('\xa0', '')

            all_images = soup.find_all('div', class_='images-view-item')
            js['images'] = []
            for image in all_images:
                url = image.find('img')['src'].replace('.jpg_50x50', '')
                js['images'].append(url)
            js['description'] = dict()
            try:
                js['description']['features'] = str(soup.find('div', class_='product-specs').contents[0])
            except AttributeError:
                js['description']['features'] = ''

            try:
                js['description']['general-information'] = str(
                    soup.find('div', class_='product-description').contents[0])
            except AttributeError:
                js['description']['general-information'] = ''
            json_text = soup.find('script', text=re.compile('window\.runParams')).contents[0]
            json_text = json_text.split('data: ')[1].split('csrfToken:')[0][:-26]
            info_js = json.loads(json_text)

            js = self.__form_variations(js=js, s_obj=info_js)

            return js

        async def parse_category(self, url: str):
            print('Get category')
            html = await self.__get_category_html(url=url)
            json_list = self.__get_info_from_category_page(html=html)
            return json_list

        async def parser_product(self, url: str):
            html = await self.__get_product_html(url=url)
            json_list = self.__get_info_from_product_page(html=html)
            return json_list

    __instance = None

    def __new__(cls):
        if not Parser.__instance:
            Parser.__instance = Parser.__Parser()
        return Parser.__instance
