import json

from tornado.web import Application, RequestHandler
from tornado.ioloop import IOLoop
from tornado import gen

from app.parser import Parser
from app.settings import LISTEN_PORT

parser = Parser()


class Category(RequestHandler):

    @gen.coroutine
    def get(self):
        if self.request.arguments.get('url'):
            url = self.request.arguments['url'][0].decode('utf-8')
            if url == '':
                self.write({'result': [],
                            'error': 'empty url',
                            'object-nums': 0})
                return
            js_ls = yield parser.parse_category(url=url)
            js = json.dumps(js_ls, ensure_ascii=False)
            self.write({'result': js,
                        'error': '',
                        'object-nums': len(js_ls)})


class Product(RequestHandler):

    @gen.coroutine
    def get(self):
        if self.request.arguments.get('url'):
            url = self.request.arguments['url'][0].decode('utf-8')
            if url == '':
                self.write({'result': [],
                            'error': 'empty url'})
                return
            js = yield parser.parser_product(url=url)
            js = json.dumps(js, ensure_ascii=False)
            self.write({'result': js,
                        'error': ''})


def make_app():
    urls = [
        ("/api/v1.0/category/", Category),
        ("/api/v1.0/product/", Product),
    ]
    return Application(urls, debug=True)


def run_server():
    app = make_app()
    app.listen(LISTEN_PORT)
    IOLoop.instance().start()
